#Supplementary Material for 
#Quantile Benchmark Dose Estimation by Wheeler, Shao and Bailer
#This code is written in R using R 3.0.1. It requires the packages 
#'quantreg' 'utils', 'stats4' and
#'statmod'.

#The calling function is QBMD and  given on the last line of the 
#file.  The data are randomly generated using the inverse-Gaussian
#distribution



## Required Package
library(quantreg)
library(utils)
library(stats4)
library(statmod)
options(warn=-1)


#########################
### Part I: Functions ###
#########################

##############################################################################
############## QR CODE 
##############################################################################

QBMD <- function(Dat,quantiles,monotone=1,samps=2000,alpha=0.1){
  alpha = alpha/2 # make it a two sided interval
  smooth.p <- qdirtySearch(Dat,quantiles,1,start=0.0010,end=1.5,0.005)
  #BMD <- computeBMD(Dat$y,Dat$x,quantiles,smooth.p,1)
  BMD <- computeBMDWG(Dat$y,Dat$x,quantiles,smooth.p,1)
  
  BMD.sim.qr.added <- (1:samps)*0
  
  ux  <- unique(Dat$x); 
  
  ttr <- length(ux)
  boot.data <- matrix(0,ncol=samps,nrow=length(Dat$y))
  for (i in 1:ttr){
    ttn = sum(Dat$x==ux[i])
    idx   <- sample(1:ttn,length(Dat$x[Dat$x==ux[i]])*samps,replace=TRUE)
    resp.i <- Dat$y[Dat$x == ux[i]]
    boot.data[Dat$x==ux[i],] <- resp.i[idx]
  }
  
  #compute the jacknife for the BCa intervals
  
  jnife.est = (1:length(Dat$x))*0
  for( i in 1:length(Dat$x)){
    jnife.est[i] =  computeBMD(Dat$y[-i],Dat$x[-i],quantiles,smooth.p,1) 
  }
  BMD.s<-apply(boot.data,2,computeBMD,x=Dat$x,taus=quantiles,lambda=smooth.p,monotone=1)
  
  U.i = (length(Dat$x)-1)*(BMD-jnife.est)
  a = sum(U.i^3)/sum(U.i^2)^(3/2)*(1/6) # estimate the acceleration
  z.0 = qnorm(sum(BMD.s < BMD)/length(BMD.s))
  z.alpha = qnorm(1-alpha)
  
  BMD.s = sort(BMD.s)
  temp = floor(pnorm(z.0 + (z.0-z.alpha)/(1-a*(z.0+z.alpha)))*samps)
  if(is.nan(temp) ){
    temp = floor(0.05*samps)
  }
  B1 = BMD.s[temp]
  
  z.alpha = qnorm(alpha)
  
  BMD.s = sort(BMD.s)
  temp = floor(pnorm(z.0 + (z.0-z.alpha)/(1-a*(z.0+z.alpha)))*samps)
  if(is.nan(temp) ){
    temp = floor(0.05*samps)
  }
  B2 = BMD.s[temp]

  return(list(BMD=BMD,BMDL = sort(c(B1,B2))))
}

computeBMD <- function(y,x,taus,lambda,monotone=1){
  
  totKnots <- length(unique(x))
  knots=seq(min(x),max(x),(max(x)-min(x))/totKnots)  
  xx <- ispline(knots,x,min(x)-0.01,max(x)+0.01)
  
  bhat = fitModel(xx,y,taus,lambda,monotone)
  b = GLOBAL_BBX%*%bhat
  
  if (monotone == 1){
    test = abs(b[1,2]-b[,1]);
    a1 = b[1,2]
  }else{
    test = abs(b[1,1]-b[,2]);
    a1 = b[1,1]; 
  }
  
  #plot(x,y)
  #lines(tx,b[,1])
  #lines(tx,b[,2])
  BMD = MTX[min(test)==test][1] 
  return(min(BMD))
} 
computeBMDWG <- function(y,x,taus,lambda,monotone=1){
  
  totKnots <- length(unique(x))
  knots=seq(min(x),max(x),(max(x)-min(x))/totKnots)  
  xx <- ispline(knots,x,min(x)-0.01,max(x)+0.01)
  
  bhat = fitModel(xx,y,taus,lambda,monotone)
  b = GLOBAL_BBX%*%bhat
  
  if (monotone == 1){
    test = abs(b[1,2]-b[,1]);
    a1 = b[1,2]
  }else{
    test = abs(b[1,1]-b[,2]);
    a1 = b[1,1]; 
  }
  
  
  tx = seq(min(x),max(x),(max(x)-min(x))/100);
  xx <- ispline(knots,tx,min(x)-0.01,max(x)+0.01)
  b = cbind(rep(1,nrow(xx)),xx)%*%bhat
  
  plot(x,y)
  lines(tx,b[,1])
  lines(tx,b[,2])
  
  BMD = MTX[min(test)==test][1]
  return(BMD)
} 

qdirtySearch <- function(Dat,quantiles,monotone,start=0.001,end=1.5,by=0.005){
  x = Dat$x
  y = Dat$y
  taus = quantiles; 
  lambda <- seq(start,end,by); 
  totKnots <- length(unique(x))
  
  knots=seq(min(x),max(x),(max(x)-min(x))/totKnots)  
  xx <- ispline(knots,x,min(x)-0.01,max(x)+0.01)
  
  
  bhat     = fitModel(xx,y,taus,lambda[1],monotone)
  for (ijk in 2:length(lambda)){
    bhat   = cbind(bhat,fitModel(xx,y,taus,lambda[ijk],monotone))
  }
  
  tres      = cbind(rep(1,length(y)),xx)%*%bhat;              
  diff      = matrix(y,ncol=ncol(tres),nrow=length(y))-tres;  
  n = length(y); 
  tauMatrix = matrix(sort(taus),ncol=length(taus)*length(lambda),nrow=length(y),byrow=TRUE)
  #note: in the following code matrix(x,nrow=length(taus)) is used to arrange the 
  #computed statistics by row
  plamM     = colSums(matrix(colSums(abs(diff) < 1e-7),nrow=length(taus)));  
  temp      = diff*(tauMatrix-(diff < 0))
  t.a3      = colSums(matrix(.5*log(n)/n*plamM,nrow=length(taus)))
  t.a1      = colSums(temp)/n
  t.a2      = log(colSums(matrix(t.a1,nrow=length(taus))))
  t.a2      = t.a2 + t.a3; 
  
  return( min(lambda[min(t.a2) == t.a2]))
  
}


fitModel <- function(x,y,taus,lambda,monotone=1){    
  
  if (length(taus)<2)
    stop("At least 2 quantile levels should be specified. If only using a single quantile, you should use rq.")
  
  taus = sort(taus)
  if (max(taus)>=1)
    stop("All quantile levels should be between 0 and 1, not including the boundary.")
  if (min(taus)<=0)
    stop("All quantile levels should be between 0 and 1, not including the boundary.")
  
  
  n = nrow(x)
  p = ncol(x)
  m = length(taus)
  pp = p+1
  
  xtemp = x
  
  x = matrix(0,nrow=n,ncol=p)
  shifts = apply(xtemp,2,min)
  scalings = rep(0,p)
  #################################################
  #rescale the matrix to be in the [0,1]^p hypercube
  #as in bondell and riech
  for (i in 1:p)
  {
    x[,i] = xtemp[,i] - shifts[i]
    scalings[i] = max(x[,i])
    x[,i] = x[,i]/scalings[i]
  }
  #################################################
  # Add the intercept is this necessary for my code
  x = cbind(rep(1,n),x)
  
  # Add the penalty terms which are the absolute differences in the beta 
  V = diag(ncol(x)); V[1,1] =0
  for (ii in 2:(ncol(V)-1)){ V[ii,ii+1] = -1;}
  V = V%*%t(V); 
  V[ncol(V),ncol(V)] = 2; 
  
  V = lambda*V[-c(1),]
  tb = matrix(rep(1-taus,nrow(x)),ncol=length(taus),byrow=TRUE);
  ta = matrix(rep(.5,length(taus)*nrow(V)),ncol=length(taus)); 
  my.mult = matrix(rbind(tb,ta)); 
  x = rbind(x,V);
  n = nrow(x)
  y = c(y,rep(0,nrow(V)))
  
  Y = rep(y, m)
  D = diag(m)
  D[lower.tri(D)] = 1 #make d a lower dimensional matrix of 1's
  X = D %x% x         #Augment the design matrix for the 'new' gamma specification
  X2 = cbind(X, -X)   #poitive and negative parts for the gammas as in Bondell, Reich, and Wang
  sX = as.matrix.csr(X2)
  K = m*pp
  R1 = matrix((diag(m) %x% rep(1,1) %x% t(c(1, rep(0, p))))[-1,],ncol=K)
  R2 = rep(1,1)
  for (j in 1:p)
  {
    R2 = cbind(R2, rep(1,1) %x% (diag(1) %x% rep(1,1)))
  }
  
  R2 = matrix((diag(m) %x% R2)[-1,],ncol=K) 
  
  ################################################################
  #Create the LHS of the constraint matrix on the gammas
  #note here one has 2K postivity constraints on the gammas
  #this is followed by the q-1 non crossing constriants
  #defined in equation (3) in Bondell Reich and Wang
  ################################################################
  getG = cbind(diag(K),-diag(K))  # recreate the gammas from the positive and negative parts
  D = diag(m)
  D[lower.tri(D)] = 1 #make d a lower dimensional matrix of 1's
  posC = monotone*(D %x% diag(pp)[-1,]) %*% getG
  #posC =  cbind(monotone*diag(K),-1*monotone*diag(K));
  sR = as.matrix.csr(rbind(diag(2*K), cbind(R1, -R2),posC))
  #r2 specifies RHS of the postitivity constraints
  
  r2 = rep(0, 2*K + (m-1)*(1)+nrow(posC))
  
  tau.v = rep(taus, each=n)
  rhs = t(X2)%*%my.mult; #t(X2)%*%(1-tau.v)
  
  coeff1 =  myrq.fit.sfnc(sX, Y, sR, r2, tau=tau.v, rhs=rhs,tmpmax=1000000)$coef
  
  
  coeff = coeff1[1:K]-coeff1[-(1:K)] #Recreate the origional Betas
  gamma.m = matrix(coeff, ncol=m)
  D = diag(m)
  D[upper.tri(D)]=1
  bhat.temp = gamma.m %*% D
  #####################################################
  ## Transform the betas back onto the origional scale.
  ##################################################### 
  transform.mat = rbind(c(1,-shifts/scalings),cbind(rep(0,p),diag(as.vector(1/scalings), nrow = length(as.vector(1/scalings)), ncol = length(as.vector(1/scalings)))))
  bhat = transform.mat%*%bhat.temp
  
  return(bhat); 
}  



myrq.fit.sfnc <- function (x, y, R, r, tau, rhs, nsubmax, tmpmax, nnzlmax, cachsz = 64, 
                           small = 1e-08, maxiter = 100, warn.mesg = TRUE) 
{
  require(quantreg)
  options(showWarnCalls=FALSE)
  y <- -y
  r <- -r
  n1 <- length(y)
  m <- x@dimension[2]
  if (n1 != x@dimension[1]) 
    stop("The design matrix A1' and response vector y are not compatible")
  n2 <- length(r)
  if (n2 != R@dimension[1]) 
    stop("The constraint matrix A2' and the constraint right-hand-side are not compatible")
  maxn1n2 <- max(n1, n2)
  u <- rep(1, length = n1)
  if(length(tau)==1)     x1 <- rep(1 - tau, length = n1)
  else x1=1-tau
  x2 <- rep(1, length = n2)
  wwm <- vector("numeric", 6 * m)
  wwm[1:m] <- rhs
  nnzx <- x@ia[x@dimension[1] + 1] - 1
  nnzR <- R@ia[R@dimension[1] + 1] - 1
  nnzdmax <- max(nnzx, nnzR)
  iwmax <- 7 * m + 3
  ao1 <- t(x)
  ao2 <- t(R)
  e <- ao1 %*% x
  g <- ao2 %*% R
  h <- e + g
  nnzemax <- e@ia[e@dimension[1] + 1] - 1
  nnzgmax <- g@ia[g@dimension[1] + 1] - 1
  nnzhmax <- h@ia[h@dimension[1] + 1] - 1
  if (missing(nnzlmax)) 
    nnzlmax <- 4 * nnzdmax
  if (missing(nsubmax)) 
    nsubmax <- nnzhmax
  if (missing(tmpmax)) 
    tmpmax <- 6 * m
  s <- u - x1
  
  chol.o <- chol(e+0.0001*diag(nrow(e)), tmpmax = tmpmax, nsubmax = nsubmax, nnzlmax = nnzlmax)
  b <- backsolve(chol.o, ao1 %*% y)
  r1 <- y - x %*% b
  z1 <- ifelse(abs(r1) < small, (r1 * (r1 > 0) + small), r1 * 
                 (r1 > 0))
  w <- z1 - r1
  z2 <- rep(1, n2)
  wwn1 <- matrix(0, n1, 10)
  wwn1[, 1] <- z1
  wwn1[, 2] <- w
  wwn2 <- matrix(0, n2, 7)
  wwn2[, 2] <- z2
  
  srqfnc.o <- .Fortran("srqfnc", n1 = as.integer(n1), m = as.integer(m), 
                       nnzx = as.integer(nnzx), x = as.double(x@ra), jx = as.integer(x@ja), 
                       ix = as.integer(x@ia), ao1 = as.double(ao1@ra), jao1 = as.integer(ao1@ja), 
                       iao1 = as.integer(ao1@ia), n2 = as.integer(n2), nnzR = as.integer(nnzR), 
                       R = as.double(R@ra), jR = as.integer(R@ja), iR = as.integer(R@ia), 
                       ao2 = as.double(ao2@ra), jao2 = as.integer(ao2@ja), iao2 = as.integer(ao2@ia), 
                       nnzdmax = as.integer(nnzdmax), d = double(nnzdmax), jd = integer(nnzdmax), 
                       id = integer(m + 1), dsub = double(nnzhmax + 1), jdsub = integer(nnzhmax + 
                                                                                          1), nnzemax = as.integer(nnzemax), e = as.double(e@ra), 
                       je = as.integer(e@ja), ie = as.integer(e@ia), nnzgmax = as.integer(nnzgmax), 
                       g = double(nnzgmax), jg = integer(nnzgmax), ig = integer(m + 
                                                                                  1), nnzhmax = as.integer(nnzhmax), h = double(nnzhmax), 
                       jh = integer(nnzhmax), ih = integer(m + 1), nsubmax = as.integer(nsubmax), 
                       lindx = integer(nsubmax), xlindx = integer(m + 1), nnzlmax = as.integer(nnzlmax), 
                       lnz = double(nnzlmax), xlnz = integer(m + 1), iw = integer(m * 
                                                                                    5), iwmax = as.integer(iwmax), iwork = integer(iwmax), 
                       xsuper = integer(m + 1), tmpmax = as.integer(tmpmax), 
                       tmpvec = double(tmpmax), maxn1n2 = as.integer(maxn1n2), 
                       ww1 = double(maxn1n2), wwm = as.double(wwm), wwn1 = as.double(wwn1), 
                       wwn2 = as.double(wwn2), cachsz = as.integer(cachsz), 
                       level = as.integer(8), x1 = as.double(x1), x2 = as.double(x2), 
                       s = as.double(s), u = as.double(u), c1 = as.double(y), 
                       c2 = as.double(r), sol = as.double(b), small = as.double(small), 
                       ierr = integer(1), maxiter = as.integer(maxiter), time = double(7), 
                       PACKAGE = "quantreg")[c("sol", "ierr", "maxiter", "time")]
  ierr <- srqfnc.o$ierr
  if (ierr == 13) 
    stop("Increase nnzh.factor")
  #  if (!(ierr == 0) && warn.mesg) 
  #   warning(sfnMessage(ierr))
  list(coef = -srqfnc.o$sol, ierr = ierr, it = srqfnc.o$maxiter, 
       time = sum(srqfnc.o$time))
}

ispline = function(knots, t,lb,ub){
  #UNTITLED Summary of this function goes here
  #   Detailed explanation goes here
  knots = c(lb, knots, ub); 
  X = matrix(0,length(t),length(knots)-2);
  j = 0; 
  
  
  for (i in 2:(length(knots) - 1)){
    h1 = knots[i-1]; 
    h2 = knots[i];  
    h3 = knots[i+1];  
    z1=(2/((h3-h1)*(h2-h1))); 
    z2=(2/((h3-h1)*(h3-h2)));      
    
    #vector if t is less than the 
    #given knot h1 through h3
    LTH1 = (t <= h1);
    LTH2 = (t <= h2); 
    LTH3 = (t <= h3); 
    
    # perform the necessary integrations for the triangle
    # distribution
    S1 = z1*(1/(j+2)*h2^(j+2)- h1/(j+1)*h2^(j+1)+ (1/(j+1)-1/(j+2))*h1^(j+2)); 
    S2 = z2*(h3/(j+1)*h3^(j+1)-1/(j+2)*h3^(j+2) + (h2^(j+2)/(j+2)-h3*h2^(j+1)/(j+1)));
    
    A1 = z1*(1/(j+2)*t^(j+2)- h1/(j+1)*t^(j+1)+ (1/(j+1)-1/(j+2))*h1^(j+2)); 
    A2 = S1 + z2*(h3/(j+1)*t^(j+1)-1/(j+2)*t^(j+2) + (h2^(j+2)/(j+2)-h3*h2^(j+1)/(j+1))); 
    
    X[,i-1] = (1-LTH1)*LTH2*A1 + (1-LTH1)*(1-LTH2)*LTH3*A2 + (1-LTH1)*(1-LTH2)*(1-LTH3)*(S1+S2); 
    
  }
  return(X);    
}

#############################################
##Data setup 
#Randomly generate a dataset as in the simulation
##############################################
get.resp.true<-function(dose,beta0,beta1,beta2){
  resp<-beta0+beta1*dose+beta2*dose^2
  return(resp)
}
###############################################
dose<-c(0,0.2,0.4,0.6,0.8,1) 
num.per.group<-45
dose.ind.temp<-rep(dose[1],num.per.group)
for (i in 2:length(dose)){
  dose.ind.temp<-c(dose.ind.temp,rep(dose[i],num.per.group))
}

dose.ind<-dose.ind.temp
beta0.true<-15
beta1.true<-3
beta2.true<-7
lambda.true<-450

sim.mean<-get.resp.true(dose.ind,15,3,7)
####
#
####
iter <-  length(unique(dose))
n <-  length(dose.ind)
resp.ind <- dose.ind*0
for (j in 1:n){
    resp.temp<-0
    resp.ind[j]<-rinvgauss(1,sim.mean[j],450)    
}

Dat <- list(y=resp.ind,x=dose.ind)

totKnots <- length(unique(dose.ind))
knots=seq(min(dose.ind),max(dose.ind),(max(dose.ind)-min(dose.ind))/totKnots) 
MTX = seq(min(dose.ind),max(dose.ind),by=(max(dose.ind)-min(dose.ind))/10000)
GLOBAL_BBX = ispline(knots,MTX,min(MTX)-0.01,max(MTX)+0.01)
GLOBAL_BBX = cbind(rep(1,length(MTX)),GLOBAL_BBX)

#compute the BMD using the QBMD method
#Here  monotone = 1 is increasing 0 is decreasing
#samps is the number of boostrap resamples and
#alpha is the value for a two  sided confidence interval
#on the BMD thus for an 0.05 level .1  should be
#specified
QBMD(Dat,c(0.8,0.9),monotone=1,samps=2000,alpha=0.1)

##############################################################################
#Code necessary to reproduce the 
#true mean curves used in the simulation
##############################################################################
#Hill Model
get.resp.true<-function(dose,beta0,beta1,beta2,beta3){
  resp<-beta0+beta1*dose^beta2/(beta3^beta2+dose^beta2)
  return(resp)
}
beta0.true<-15
beta1.true<-11
beta2.true<-3
beta3.true<-0.5

# Polynomial Model
get.resp.true<-function(dose,beta0,beta1,beta2){
  resp<-beta0+beta1*dose+beta2*dose^2
  return(resp)
}
beta0.true<-15
beta1.true<-3
beta2.true<-7

# Power Model  
get.resp.true<-function(dose,beta0,beta1,beta2){
  resp<-beta0+beta1*dose^beta2
  return(resp)
}

beta0.true<-15
beta1.true<-10
beta2.true<-0.6


#I-Spline Conditions 4 and 5
get.true.resp<-function(bhat.true, dlevels){
  len<-length(bhat.true)
  knot.num<-len-2
  knots=seq(min(dlevels),max(dlevels),(max(dlevels)-min(dlevels))/knot.num)  
  X = ispline(knots,dlevels,min(dlevels)-0.01,max(dlevels)+0.01)
  tt <- cbind(rep(1,length(dlevels)),X) 
  b <- tt%*%bhat.true
  return(as.vector(b))
}
#I-Spline Condition 1
bhat.true<-c(1.511861e+01,3.702496e-09,1.183689e-08,6.164300e-02,2.662945e-01,6.404823e-01,1.014670e+00,1.388858e+00,1.763046e+00,2.137233e+00,1.424822e+00,7.836523e-01)
#I-Spline Condition 2
bhat.true<-c(14.79634943,0.90270352,1.64127881,1.63649988,1.63172076,1.62694149,1.31423915,0.83471280,0.35518645,0.11112164,0.07408080,0.04074431)


